// 3D Printed Sink Strainer
// Ed Nisley - KE4ZNU - Sept 2011

Layout = "Fit";					// Build Fit

//- Extrusion parameters - must match reality!

ThreadThick = 0.33;
ThreadWidth = 2.0 * ThreadThick;

function IntegerMultiple(Size,Unit) = Unit * ceil(Size / Unit);

Protrusion = 0.1;
HoleWindage = 0.0;

//- Strainer sizes

PlateThick = IntegerMultiple(4.0,ThreadThick);
PlateDia = 80.0;
PlateRadius = PlateDia/2;
PlateSides = 64;

HoleDia = 4.75;
HoleSides = 8;

Ring1Dia = PlateDia - 2.5 * HoleDia;
Ring2Dia = Ring1Dia - 3.5 * HoleDia;
Ring3Dia = Ring2Dia - 3.5 * HoleDia;

HandleDia = IntegerMultiple(9.5,ThreadWidth);
HandleLength = 20.0;
HandleSides = 8;

HandleStubDia = IntegerMultiple(5.0,ThreadWidth);
HandleStubLength = IntegerMultiple(0.3*PlateThick,ThreadThick);

HandleBuildRadius = PlateRadius + HandleDia;



//----------------------
// Useful routines

//- Enlarge holes to prevent geometric shrinkage
// based on nophead's polyholes
// http://hydraraptor.blogspot.com/2011/02/polyholes.html
// http://www.thingiverse.com/thing:6118

module PolyCyl(Dia,Height,ForceSides=0) {

  Sides = (ForceSides != 0) ? ForceSides : (ceil(Dia) + 2);

  FixDia = Dia / cos(180/Sides);

  cylinder(r=(FixDia + HoleWindage)/2,
           h=Height,
		   $fn=Sides);
}

//- Put non-printing reference grid on build surface

module ShowPegGrid(Space = 10.0,Size = 1.0) {

  Range = floor(50 / Space);

	for (x=[-Range:Range])
	  for (y=[-Range:Range])
		translate([x*Space,y*Space,Size/2])
		  %cube(Size,center=true);

}

//- Create a handle

module Handle() {

  union() {
	cylinder(r=HandleDia/2,h=HandleLength,$fn=HandleSides);
	translate([0,0,HandleLength])
	  cylinder(r=HandleStubDia/2,h=HandleStubLength,$fn=HandleSides);
  }
}

//- Create a ring of holes
//  Equally spaced with as many as fit into the ring

module RingHoles(RingDia,HoleDia,Thick) {

  Num = floor(90/asin(HoleDia/RingDia));

  for(n=[0:(Num-1)]) {
	rotate([0,0,n*360/Num])
	  translate([RingDia/2,0,-Protrusion])
		rotate([0,0,180/HoleSides])
		  PolyCyl(HoleDia,(Thick + 2*Protrusion),HoleSides);
  }

}

//- Create the strainer plate

module StrainerPlate() {

  difference() {

	cylinder(r=PlateRadius,h=PlateThick,$fn=PlateSides);

	translate([0,0,-Protrusion])
	  PolyCyl(HandleStubDia,(PlateThick + 2*Protrusion));

	RingHoles(Ring1Dia,HoleDia,PlateThick);
	RingHoles(Ring2Dia,HoleDia,PlateThick);
	RingHoles(Ring3Dia,HoleDia,PlateThick);

  }
}

//---------------
//- Put it all together

if (Layout == "Fit") {
  StrainerPlate();
  translate([0,0,-HandleLength])
	Handle();
  translate([0,0,(PlateThick + HandleLength)])
	rotate([180,0,0])
	  Handle();
}

if (Layout == "Build") {
  ShowPegGrid();
  StrainerPlate();
  for (a=[35,55]) {
	translate([HandleBuildRadius*cos(a),HandleBuildRadius*sin(a),0])
	  Handle();
  }
}
